<?php
/**
 * Suppress "error - 0 - No summary was found for this file" on phpdoc generation
 *
 * @package WPDataProjects
 */

namespace WPDataProjects\List_Table {

	use WPDataAccess\List_Table\WPDA_List_View;
	use WPDataProjects\Data_Dictionary\WPDP_List_Columns;
	use WPDataProjects\Project\WPDP_Project;

	/**
	 * Class WPDP_List_View
	 *
	 * @package WPDataProjects\List_Table
	 * @author  Peter Schulz
	 * @since   2.0.0
	 */
	class WPDP_List_View extends WPDA_List_View {

		/**
		 * @var null|string
		 */
		protected $project_id = null;
		/**
		 * @var null
		 */
		protected $page_id = null;

		/**
		 * @var null
		 */
		protected $title;
		/**
		 * @var null
		 */
		protected $subtitle;

		/**
		 * @var
		 */
		protected $mode;

		/**
		 * WPDP_List_View constructor.
		 *
		 * @param array $args
		 */
		public function __construct( array $args = [] ) {
			if ( isset( $args['project_id'] ) ) {
				$this->project_id = sanitize_text_field( wp_unslash( $args['project_id'] ) );
			} elseif ( isset( $_REQUEST['tab'] ) && 'tables' === $_REQUEST['tab'] ) {
				$this->project_id = 'wpda_sys_tables';
			}
			if ( isset( $args['page_id'] ) ) {
				$this->page_id = sanitize_text_field( wp_unslash( $args['page_id'] ) );
			}

			$this->project  = new WPDP_Project( $this->project_id, $this->page_id );
			$this->title    = $this->project->get_title();
			$this->subtitle = $this->project->get_subtitle();
			$this->mode     = $this->project->get_mode();

			$args['title']    = ( null === $this->title || '' === $this->title ) ? null : $this->title;
			$args['subtitle'] = $this->subtitle;

			parent::__construct( $args );

			$lable_type = 'listtable';
			if (
				'edit' === $this->action ||
				'new'  === $this->action ||
				'view' === $this->action
			) {
				$lable_type = 'tableform';
			}

			// Overwrite list columns.
			$this->wpda_list_columns = new WPDP_List_Columns( $this->schema_name, $this->table_name, $lable_type );

			// Overwrite column header text.
			$this->column_headers = isset( $args['column_headers'] ) ? $args['column_headers'] : '';
		}

		/**
		 * Overwrite method.
		 */
		protected function display_list_table() {

			$this->wpda_list_table = new WPDP_List_Table(
				[
					'schema_name'       => $this->schema_name,
					'table_name'        => $this->table_name,
					'wpda_list_columns' => $this->wpda_list_columns,
					'column_headers'    => $this->column_headers,
					'title'				=> $this->title,
					'subtitle'			=> $this->subtitle,
					'mode'				=> $this->mode,
				]
			);

			$this->wpda_list_table->show();

		}

	}

}